package com.sztzjy.digital_credit.service.impl;

import com.sztzjy.digital_credit.config.Constant;
import com.sztzjy.digital_credit.entity.*;
import com.sztzjy.digital_credit.mapper.StuScoreCenterMapper;
import com.sztzjy.digital_credit.mapper.StuUserMapper;
import com.sztzjy.digital_credit.mapper.TchModuleWeithMapper;
import com.sztzjy.digital_credit.service.StuScoreCenterService;
import com.sztzjy.digital_credit.util.BigDecimalUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;

/**
 * @author tz
 * @date 2024/4/23 10:22
 */
@Service
public class StuScoreCenterServiceImpl implements StuScoreCenterService {
    @Resource
    StuScoreCenterMapper stuScoreCenterMapper;
    @Resource
    StuUserMapper stuUserMapper;
    @Resource
    TchModuleWeithMapper tchModuleWeithMapper;
    @Resource
    BigDecimalUtils bigDecimalUtils;
    @Override
    public List<StuScoreCenter> getScore(String userId, String module) {
        Map<String, List<StuScoreCenter>> totalScoreCalculation = this.getTotalScoreCalculation(userId);
        if(module.equals(Constant.COMPREHENSIVE_CASE_SCORE)){
            //取到对应值
            List<StuScoreCenter> stuScoreCenters = totalScoreCalculation.get(module);
            // TODO: 2024/4/24 计算总成绩

            //学习项目（在成绩前面加个总）
            StringBuilder stringBuilder=new StringBuilder(module);
            String string = stringBuilder.insert(4, "总").toString();

            StuScoreCenter stuScoreCenter = this.calculateTotalScore(stuScoreCenters, string);
            stuScoreCenters.add(stuScoreCenter);
            return stuScoreCenters;
        }else if(module.equals(Constant.SCORE_OVERVIEW) || module.isEmpty()){
            module="成绩总览";
            //查询用户信息，插入模块排名
            StuUser stuUser = stuUserMapper.selectByPrimaryKey(userId);

            List<StuScoreCenter> list=new ArrayList<>();
            for(Map.Entry<String,List<StuScoreCenter>> stringListEntry:totalScoreCalculation.entrySet()){

                List<StuScoreCenter> value = stringListEntry.getValue();

                //遍历键，根据键取到成绩排名
                String key = stringListEntry.getKey();


                if(value.isEmpty()){
                    StuScoreCenter stuScoreCenter=new StuScoreCenter();
                    String substring = key.substring(0, 4);
                    stuScoreCenter.setLearningProjects(substring);

                    list.add(stuScoreCenter);
                }else {
                    String substring = key.substring(0, 4);
                    StuScoreCenter stuScoreCenter = this.calculateTotalScore(value, substring);

                    if(key.equals(Constant.CREDIT_PORTRAIT_SCORE)){ //征信画像排名
                        stuScoreCenter.setClassRanking(stuUser.getCreditPortraitRank());
                    }
                    if(key.equals(Constant.PERSONAL_CREDIT_SCORE)){ //个人征信排名
                        stuScoreCenter.setClassRanking(stuUser.getPersonalCreditRank());
                    }
                    if(key.equals(Constant.CORPORATE_CREDIT_SCORE)){ //企业征信排名
                        stuScoreCenter.setClassRanking(stuUser.getCorporateCreditRank());
                    }
                    if(key.equals(Constant.COMPREHENSIVE_CASE_SCORE)){ //综合案例排名
                        stuScoreCenter.setClassRanking(stuUser.getComprehensiveCaseRank());
                    }
                    list.add(stuScoreCenter);
                }
            }


            StuScoreCenter stuScoreCenter=new StuScoreCenter();
            //计算总成绩

            //学习项目
            stuScoreCenter.setLearningProjects("总成绩");

            //实训时长和实训得分
            int duration=0;
            BigDecimal score=new BigDecimal(0);
            for (int i = 0; i < list.size(); i++) {
                //实训时长
                if(list.get(i).getTrainingDuration()==null){

                }else {
                    Integer trainingDuration = list.get(i).getTrainingDuration();
                    duration=duration+trainingDuration;
                }

                //实训得分
                if(list.get(i).getTrainingScore()==null){

                }else {
                    BigDecimal trainingScore = list.get(i).getTrainingScore();
                    score=bigDecimalUtils.add(score.toString(),trainingScore.toString());
                }
            }
            stuScoreCenter.setTrainingDuration(duration);
            stuScoreCenter.setTrainingScore(score);

            //总成绩排名
            stuScoreCenter.setClassRanking(stuUser.getTotalRank());

            list.add(stuScoreCenter);

            return list;
        }else {
            // TODO: 2024/4/28 如果是征信画像、个人征信、企业征信
            return null;
        }
    }

    /**
     * 实训时长统计
     * @param userId 用户ID
     * @param module 模块
     * @param viewingTime 观看时间
     */
    @Override
    public void trainingDurationStatistics(String userId, String module, Integer viewingTime,String ascription) {
         StuScoreCenterExample example=new StuScoreCenterExample();
         example.createCriteria().andUserIdEqualTo(userId).andModuleEqualTo(module).andAscriptionEqualTo(ascription);
        List<StuScoreCenter> stuScoreCenters = stuScoreCenterMapper.selectByExample(example);
        if(stuScoreCenters.isEmpty()){
            StuScoreCenter stuScoreCenter=new StuScoreCenter();
            Integer uuid = UUID.randomUUID().toString().replaceAll("-", "").hashCode();
            uuid = uuid < 0 ? -uuid : uuid;//String.hashCode() 值会为空
            stuScoreCenter.setId(uuid);
            stuScoreCenter.setUserId(userId);
            stuScoreCenter.setLearningProjects(module);
            stuScoreCenter.setModule(module);
            stuScoreCenter.setAscription(ascription);
            stuScoreCenter.setCompletionStatus("未完成");
            stuScoreCenter.setTrainingDuration(viewingTime);
            stuScoreCenter.setTrainingTotalScore(BigDecimal.valueOf(100));
            stuScoreCenter.setTrainingScoreOne(BigDecimal.valueOf(0));
            stuScoreCenter.setTrainingScoreTwo(BigDecimal.valueOf(0));
            stuScoreCenter.setTrainingScoreThree(BigDecimal.valueOf(0));
            stuScoreCenter.setNumberOfTraining(0);

            stuScoreCenterMapper.insert(stuScoreCenter);
        }else {
            StuScoreCenter stuScoreCenter = stuScoreCenters.get(0);
            BigDecimal add = bigDecimalUtils.add(String.valueOf(viewingTime), String.valueOf(stuScoreCenter.getTrainingDuration()));
            stuScoreCenter.setTrainingDuration(Integer.valueOf(String.valueOf(add)));

            stuScoreCenterMapper.updateByPrimaryKey(stuScoreCenter);
        }
    }


    /**
     * 根据模块封装
     * @param userId
     * @return
     */
    public Map<String,List<StuScoreCenter>> getTotalScoreCalculation(String userId){
        StuScoreCenterExample scoreCenterExample=new StuScoreCenterExample();
        scoreCenterExample.createCriteria().andUserIdEqualTo(userId);
        List<StuScoreCenter> stuScoreCenters = stuScoreCenterMapper.selectByExample(scoreCenterExample);

        //征信画像总成绩
        List<StuScoreCenter> creditPortraits=new ArrayList<>();
        //个人征信总成绩
        List<StuScoreCenter> personalCredits =new ArrayList<>();
        //企业征信总成绩
        List<StuScoreCenter> corporateCredits=new ArrayList<>();
        //综合案例总成绩
        List<StuScoreCenter> comprehensiveCases=new ArrayList<>();

        Map<String,List<StuScoreCenter>> map=new HashMap<>();
//        int a=0;int b=0;int c=0;
        for (int i = 0; i < stuScoreCenters.size(); i++) {
            StuScoreCenter scoreCenter = stuScoreCenters.get(i);

            if(scoreCenter.getAscription().equals(Constant.CREDIT_PORTRAIT)){
                creditPortraits.add(scoreCenter);
            }

            if(scoreCenter.getAscription().equals(Constant.PERSONAL_CREDIT)){
                personalCredits.add(scoreCenter);
            }

            if(scoreCenter.getAscription().equals(Constant.CORPORATE_CREDIT)){
                corporateCredits.add(scoreCenter);
            }

            if(scoreCenter.getAscription().equals(Constant.COMPREHENSIVE_CASE)){
                comprehensiveCases.add(scoreCenter);
            }
        }
        String[] creditPortrait={"贷款案例","个人信息与违约","还款行为与违约"};
        String[] personalCredit={"影响因素分析","征信优化分析","征信评级模型"};
        String[] corporateCredit={"影响因素分析","征信优化分析","征信评级模型"};
        String[] comprehensiveCase={"用户画像","个人征信","企业征信"};

        commonMethods(userId, creditPortraits, creditPortrait,"征信画像"); //征信画像


        commonMethods(userId, personalCredits, personalCredit,"个人征信"); //个人征信


        commonMethods(userId, corporateCredits, corporateCredit,"企业征信"); //企业征信


        commonMethods(userId,comprehensiveCases,comprehensiveCase,"综合案例"); //综合案例


        //封装到map
        map.put(Constant.CREDIT_PORTRAIT_SCORE,creditPortraits);
        map.put(Constant.PERSONAL_CREDIT_SCORE,personalCredits);
        map.put(Constant.CORPORATE_CREDIT_SCORE,corporateCredits);
        map.put(Constant.COMPREHENSIVE_CASE_SCORE,comprehensiveCases);

        return map;
    }

    /**
     * 统一返回形式
     * @param userId 用户ID
     * @param creditPortraits 对应模块成绩中心数据
     * @param creditPortrait 模块数字
     * @param ascription 归属大模块
     */
    private void commonMethods(String userId, List<StuScoreCenter> creditPortraits, String[] creditPortrait,String ascription) {
        if(!creditPortraits.isEmpty()){
            for (String s : creditPortrait) {
                for (int j = 0; j < creditPortraits.size(); j++) {
                    if (s.equals(creditPortraits.get(j).getModule())) {
                        break;
                    }
                    StuScoreCenter stuScoreCenter = new StuScoreCenter();
                    stuScoreCenter.setLearningProjects(s);
                    stuScoreCenter.setTrainingTotalScore(BigDecimal.valueOf(100));
                    stuScoreCenter.setModule(s);
                    stuScoreCenter.setUserId(userId);
                    stuScoreCenter.setAscription(ascription);
                    stuScoreCenter.setCompletionStatus("未完成");
                    creditPortraits.add(stuScoreCenter);
                }
            }
        }else {
            for (String s : creditPortrait) {
                StuScoreCenter stuScoreCenter = new StuScoreCenter();
                stuScoreCenter.setLearningProjects(s);
                stuScoreCenter.setTrainingTotalScore(BigDecimal.valueOf(100));
                stuScoreCenter.setModule(s);
                stuScoreCenter.setUserId(userId);
                stuScoreCenter.setAscription(ascription);
                stuScoreCenter.setCompletionStatus("未完成");
                creditPortraits.add(stuScoreCenter);
            }
        }
    }

    /**
     * 更新权重信息
     * @param stuScoreCenters
     */
    public void updateWeightInfo(List<StuScoreCenter> stuScoreCenters){
        StuUser stuUser = stuUserMapper.selectByPrimaryKey(stuScoreCenters.get(0).getUserId());
        TchModuleWeithExample example=new TchModuleWeithExample();
        example.createCriteria().andSchoolIdEqualTo(stuUser.getSchoolId());
        List<TchModuleWeith> tchModuleWeiths = tchModuleWeithMapper.selectByExample(example);

        if(tchModuleWeiths.isEmpty()){
            TchModuleWeith tchModuleWeith=new TchModuleWeith(stuUser.getSchoolId());
            tchModuleWeithMapper.insert(tchModuleWeith);
            tchModuleWeiths.add(tchModuleWeith);
        }

        TchModuleWeith tchModuleWeith = tchModuleWeiths.get(0);

        for (int i = 0; i < stuScoreCenters.size(); i++) {
            StuScoreCenter stuScoreCenter = stuScoreCenters.get(i);
            if(stuScoreCenter.getModule().equals("贷款案例")){
                stuScoreCenter.setScoreWeight(tchModuleWeith.getLoanCasesWeight());
            }
            if(stuScoreCenter.getModule().equals("个人情况与违约")){
                stuScoreCenter.setScoreWeight(tchModuleWeith.getPerSituationWeiht());
            }
            if(stuScoreCenter.getModule().equals("还款行为与违约")){
                stuScoreCenter.setScoreWeight(tchModuleWeith.getRepaymentBehaviorWeiht());
            }

            if(stuScoreCenter.getModule().equals("影响因素与分析")){
                if(stuScoreCenter.getAscription().equals("个人征信")){
                    stuScoreCenter.setScoreWeight(tchModuleWeith.getPerInfluenceFactorWeith());
                }else {
                    stuScoreCenter.setScoreWeight(tchModuleWeith.getEntInfluenceFactorWeith());
                }
            }
            if(stuScoreCenter.getModule().equals("征信优化与分析")){
                if(stuScoreCenter.getAscription().equals("个人征信")){
                    stuScoreCenter.setScoreWeight(tchModuleWeith.getPerCreditOptimizationWeith());
                }else {
                    stuScoreCenter.setScoreWeight(tchModuleWeith.getEntCreditOptimizationWeith());
                }
            }
            if(stuScoreCenter.getModule().equals("征信评级模型")){
                if(stuScoreCenter.getAscription().equals("个人征信")){
                    stuScoreCenter.setScoreWeight(tchModuleWeith.getPerCreditRatingWeith());
                }else {
                    stuScoreCenter.setScoreWeight(tchModuleWeith.getEntCreditRatingWeith());
                }
            }

            if(stuScoreCenter.getModule().equals("用户画像")){
                stuScoreCenter.setScoreWeight(tchModuleWeith.getCaseUserProfileWeith());
            }
            if(stuScoreCenter.getModule().equals("个人征信")){
                stuScoreCenter.setScoreWeight(tchModuleWeith.getCasePersonalCreditWeith());
            }
            if(stuScoreCenter.getModule().equals("企业征信")){
                stuScoreCenter.setScoreWeight(tchModuleWeith.getCaseCorporateCreditWeith());
            }

            stuScoreCenterMapper.updateByPrimaryKey(stuScoreCenter);
        }
    }

    /**
     * 计算每个模块总成绩
     * @param stuScoreCenters
     * @param module
     * @return
     */
    public StuScoreCenter calculateTotalScore(List<StuScoreCenter> stuScoreCenters,String module){
        StuScoreCenter stuScoreCenter=new StuScoreCenter();


        stuScoreCenter.setLearningProjects(module);

        //实训时长（累加）

        //更新权重信息
        this.updateWeightInfo(stuScoreCenters);

        int duration=0;
        BigDecimal score=new BigDecimal(0);
        for (int i = 0; i < stuScoreCenters.size(); i++) {
            //取出实训时长累加
            if(stuScoreCenters.get(i).getTrainingDuration()==null){

            }else {
                Integer trainingDuration = stuScoreCenters.get(i).getTrainingDuration();
                duration=duration+trainingDuration;
            }

            //取出实训得分，并乘权重再累加
            if(stuScoreCenters.get(i).getTrainingScore()==null){

            }else {
                BigDecimal trainingScore = stuScoreCenters.get(i).getTrainingScore();
                BigDecimal scoreWeight = stuScoreCenters.get(i).getScoreWeight();

                String mul = bigDecimalUtils.mul(trainingScore.toString(), scoreWeight.toString(), 2);
                score=bigDecimalUtils.add(score.toString(),mul);
            }

        }
        stuScoreCenter.setTrainingDuration(duration);

        //完成情况
        int a=0;
            for (int i = 0; i < stuScoreCenters.size(); i++) {
                StuScoreCenter stuScoreCenter1 = stuScoreCenters.get(i);
                if(stuScoreCenter1.getCompletionStatus().equals("已完成")){
                    a++;
                }
            }
            if(a==3){
                stuScoreCenter.setCompletionStatus("已完成");
            }else {
                stuScoreCenter.setCompletionStatus("未完成");
            }


        //实训得分
        stuScoreCenter.setTrainingScore(score);

        return stuScoreCenter;
    }
}
